<?php

namespace App\Controller\Admin\Traits\Ajax;

trait RelateAjaxTrait
{
    /**
     * $config.
     *
     * @var array
     */
    protected $config;

    /**
     * 取得分類商品資料
     */
    public function ajax_relate_search()
    {
        $target_cofig = config($this->config['target_relate_config']);
        $target_table = $target_cofig['table'];
        $target_class_table = $target_cofig['class_table'] ?? '';

        // 已經關聯的
        $now_relate_pid = \DB::table($this->config['relate_table'])->where([['pid', \Request::query('pid')], ['del', 0]])->pluck('relate_pid')->toArray();
        if ($target_table == $this->config['table']) {
            $now_relate_pid = array_merge($now_relate_pid, [(int) \Request::query('pid')]);
        }

        if (!empty($target_class_table) && $this->config['target_relate_group_set']) {
            // 分類
            $class = \DB::table($target_class_table)->where([['del', 0]])->orderBy('no', 'desc')->orderBy('id', 'asc')->get()->keyBy('id');

            $data = \DB::table($target_table)
                       ->where([['del', 0]])
                       ->whereNotIn('id', $now_relate_pid)
		               ->orderBy('no', 'desc')
		               ->orderBy('id', 'asc')
		               ->get()
		               ->groupBy('class_id')
		               ->transform(function ($row, $key) use ($class) {
		                    $class_name = '';
		                    $now_class_id = $key;

		                    do {
		                        if ($class[$now_class_id]['class_id'] == 0) {
		                            $class_name = $class[$now_class_id]['name'] . $class_name;
		                        } else {
		                            $class_name = ' > ' . $class[$now_class_id]['name'] . $class_name;
		                        }

		                        $now_class_id = $class[$now_class_id]['class_id'];
		                    } while ($now_class_id != 0);

		                    $row['class_name'] = $class_name;

		                    return $row;
		               });
        } else {
            $data = \DB::table($target_table)
	                   ->when(!empty($target_class_table), function ($query) {
	                       return $query->where([['class_id', 0]]);
	                   })
	                   ->where([['del', 0]])
	                   ->whereNotIn('id', array_merge($now_relate_pid, [(int) \Request::query('pid')]))
	                   ->orderBy('no', 'desc')
	                   ->orderBy('id', 'asc')
	                   ->get();
        }

        $need_group = $this->config['target_relate_group_set'] ? true : false;

        $html = \View::make("admin.{$this->config['folder']}.relate_options", compact('data', 'class', 'need_group'))->render();
        return \JsonResponse::create(['html' => $html])->send();
    }

    /**
     * 取得關聯
     */
    public function ajax_relate_get($pid)
    {
        if ($pid == 0) {
            // 新增時 顯示暫存
            $where = [['r.session_id', \Session::getId()], ['r.pid', $pid]];
        } else {
            $where = [['r.pid', $pid]];
        }

        $target_cofig = config($this->config['target_relate_config']);
        $target_table = $target_cofig['table'];

        $data = \DB::table($this->config['relate_table'] . ' as r')
	               ->leftJoin($target_table . ' as p', 'p.id', 'r.relate_pid')
	               ->select('r.id', 'r.no', 'p.name', 'p.pic')
	               ->where($where)
	               ->orderBy('id', 'asc')
	               ->get();

        $html = \View::make("admin.{$this->config['folder']}.relate", compact('data'))->render();

        return \JsonResponse::create(['html' => $html])->send();
    }

    /**
     * 新增關聯
     */
    public function ajax_relate_add($pid)
    {
        $message = ['success' => '', 'error' => ''];

        $data = [];

        foreach (\Request::input('relate_pid') as $key => $value) {
            if (\DB::table($this->config['relate_table'])->insert(['pid' => $pid, 'relate_pid' => $value, 'session_id' => \Session::getId()])) {
                $message['success'] .= $value . " 新增成功\n";
            } else {
                $message['error'] .= $value . "新增失敗\n";
            }
        }

        return \JsonResponse::create($message)->send();
    }

    /**
     * 刪除關聯
     */
    public function ajax_relate_del($pid, $id)
    {
        $message = ['success' => '', 'error' => ''];

        if (\DB::table($this->config['relate_table'])->where('id', $id)->delete()) {
            $message['success'] = '刪除成功';
        } else {
            $message['error'] = '刪除失敗';
        }

        return \JsonResponse::create($message)->send();
    }
}